module NetSuite
  module Records
    class SerializedAssemblyItem
      include Support::Fields
      include Support::RecordRefs
      include Support::Records
      include Support::Actions
      include Namespaces::ListAcct

      actions :get, :get_list, :add, :delete, :search, :update, :upsert

      fields :alternate_demand_source_item,
        :asset_account,
        :auto_lead_time,
        :auto_preferred_stock_level,
        :auto_reorder_point,
        :available_to_partners,
        :average_cost,
        :bill_exch_rate_variance_acct,
        :billing_schedule,
        :bill_price_variance_acct,
        :bill_qty_variance_acct,
        :build_entire_assembly,
        :build_time,
        :klass,
        :cogs_account,
        :cost,
        :cost_category,
        :cost_estimate,
        :cost_estimate_type,
        :costing_method,
        :costing_method_display,
        :country_of_manufacture,
        :created_date,
        :create_revenue_plans_on,
        :custom_form,
        :default_item_ship_method,
        :default_return_cost,
        :default_revision,
        :deferred_revenue_account,
        :demand_modifier,
        :demand_source,
        :demand_time_fence,
        :department,
        :description,
        :display_name,
        :dont_show_price,
        :dropship_expense_account,
        :effective_bom_control,
        :enforce_min_qty_internally,
        :exclude_from_sitemap,
        :featured_description,
        :fixed_lot_size,
        :gain_loss_account,
        :handling_cost,
        :handling_cost_units,
        :hazmat_hazard_class,
        :hazmat_id,
        :hazmat_item_units,
        :hazmat_item_units_qty,
        :hazmat_packing_group,
        :hazmat_shipping_name,
        :include_children,
        :interco_cogs_account,
        :interco_income_account,
        :invt_classification,
        :invt_count_interval,
        :is_donation_item,
        :is_gco_compliant,
        :is_hazmat_item,
        :is_inactive,
        :is_online,
        :is_phantom,
        :is_special_work_order_item,
        :issue_product,
        :is_taxable,
        :item_carrier,
        :item_id,
        :item_number_options_list,
        :item_revenue_category,
        :last_invt_count_date,
        :last_modified_date,
        :last_purchase_price,
        :lead_time,
        :location,
        :locations_list,
        :manufacturer,
        :manufactureraddr1,
        :manufacturer_city,
        :manufacturer_state,
        :manufacturer_tariff,
        :manufacturer_tax_id,
        :manufacturer_zip,
        :match_bill_to_receipt,
        :matrix_item_name_template,
        :max_donation_amount,
        :meta_tag_html,
        :minimum_quantity,
        :minimum_quantity_units,
        :mpn,
        :mult_manufacture_addr,
        :nex_tag_category,
        :next_invt_count_date,
        :no_price_message,
        :offer_support,
        :on_special,
        :out_of_stock_behavior,
        :out_of_stock_message,
        :overall_quantity_pricing_type,
        :page_title,
        :parent,
        :preference_criterion,
        :preferred_location,
        :preferred_stock_level,
        :preferred_stock_level_days,
        :preferred_stock_level_units,
        :prices_include_tax,
        :pricing_group,
        :pricing_matrix,
        :print_items,
        :prod_price_variance_acct,
        :prod_qty_variance_acct,
        :producer,
        :product_feed_list,
        :purchase_description,
        :purchase_order_amount,
        :purchase_order_quantity,
        :purchase_order_quantity_diff,
        :purchase_price_variance_acct,
        :purchase_tax_code,
        :purchase_unit,
        :quantity_available,
        :quantity_back_ordered,
        :quantity_committed,
        :quantity_on_hand,
        :quantity_on_hand_units,
        :quantity_on_order,
        :quantity_pricing_schedule,
        :rate,
        :receipt_amount,
        :receipt_quantity,
        :receipt_quantity_diff,
        :related_items_description,
        :reorder_multiple,
        :reorder_point,
        :reorder_point_units,
        :reschedule_in_days,
        :reschedule_out_days,
        :revenue_allocation_group,
        :revenue_recognition_rule,
        :rev_rec_schedule,
        :round_up_as_component,
        :safety_stock_level,
        :safety_stock_level_days,
        :safety_stock_level_units,
        :sales_tax_code,
        :sale_unit,
        :schedule_b_code,
        :schedule_b_number,
        :schedule_b_quantity,
        :scrap_acct,
        :search_keywords,
        :seasonal_demand,
        :serial_numbers,
        :ship_individually,
        :ship_package,
        :shipping_cost,
        :shipping_cost_units,
        :shopping_dot_com_category,
        :shopzilla_category_id,
        :show_default_donation_amount,
        :sitemap_priority,
        :soft_descriptor,
        :specials_description,
        :stock_description,
        :stock_unit,
        :store_description,
        :store_detailed_description,
        :store_display_image,
        :store_display_name,
        :store_display_thumbnail,
        :store_item_template,
        :supply_lot_sizing_method,
        :supply_replenishment_method,
        :supply_time_fence,
        :supply_type,
        :tax_schedule,
        :total_value,
        :track_landed_cost,
        :transfer_price,
        :unbuild_variance_account,
        :units_type,
        :upc_code,
        :url_component,
        :use_bins,
        :use_component_yield,
        :use_marginal_rates,
        :vendor_name,
        :vsoe_deferral,
        :vsoe_delivered,
        :vsoe_permit_discount,
        :vsoe_price,
        :vsoe_sop_group,
        :weight,
        :weight_unit,
        :weight_units,
        :wip_acct,
        :wip_variance_acct

      record_refs :income_account

      # accountingBookDetailList	ItemAccountingBookDetailList
      # binNumberList	InventoryItemBinNumberList
      # itemOptionsList	ItemOptionsList
      # itemShipMethodList	RecordRefList
      # itemVendorList	ItemVendorList
      # memberList	ItemMemberList
      # numbersList	SerializedInventoryItemNumbersList
      # presentationItemList	PresentationItemList
      # siteCategoryList	SiteCategoryList
      # translationsList	TranslationList

      field :subsidiary_list, RecordRefList
      field :custom_field_list, CustomFieldList

      attr_reader :internal_id
      attr_accessor :external_id

      def initialize(attributes = {})
        @internal_id = attributes.delete(:internal_id) || attributes.delete(:@internal_id)
        @external_id = attributes.delete(:external_id) || attributes.delete(:@external_id)
        initialize_from_attributes_hash(attributes)
      end

      def self.search_class_name
        "Item"
      end
    end
  end
end
